#version 410 core

in vec3 vertexPosition;
in vec3 vertexNormal;
in vec2 vertexUVCoord;
layout(location=0) out vec4 fragmentColor;

vec4 lightColor = vec4(1.0, 1.0, 1.0, 1.0);
//vec4 ambientColor = vec4(0.0125, 0.0125, 0.0125, 1.0);
vec4 ambientColor = vec4(0.025, 0.025, 0.025, 1.0);

uniform sampler2D tex;

uniform vec3 cameraPosition;
uniform vec3 lightPosition = vec3(0.0, 1500.0, 0.0);

uniform int hasTexture = 0;
uniform int hasDiffuseColor = 0;
uniform int hasSpecularColor = 0;
uniform vec4 colorDiffuse;
uniform vec4 colorSpecular;
uniform float shininess = 0.0;

uniform vec3 spotlightPosition;
uniform vec3 spotlightDirection;
uniform vec3 spotlightColor;
uniform int night;

struct SpotLight {
    //Positions- und Beleuchtungs-Angaben
    vec3 position;  
    vec3 direction;
    vec3 color;

    //Gren der inneren und usseren Beleuchtungskreise
    float innerCone;  //cos(rad(wert))
    float outerCone; //cos(rad(wert))

    //Faktoren fr Lichtabschwchung auf Distanz
    float constant;
    float linear;
    float quadratic;
}; 

SpotLight spotLight = SpotLight(spotlightPosition, //position
                                spotlightDirection, //direction
                                spotlightColor,  //color
                                0.976, 0.953, //innerCone = cos(rad(12.5)), outerCone = cos(rad(17.5))
                                1.0, 0.004, 0.000007); //Lichtabschwaechung

void main()
{
    vec4 diffuseColor = vec4(1.0, 1.0, 1.0, 1.0);
    if(hasDiffuseColor == 1)
    {
        diffuseColor = colorDiffuse;
    }

    vec4 specularColor = vec4(0.0, 0.0, 0.0, 1.0);
    float shine = 1.0;
    if(hasSpecularColor == 1)
    {
        shine = shininess;
        specularColor = colorSpecular;
    }

    vec3 normal = normalize(vertexNormal);
    vec3 lightVector = normalize(lightPosition - vertexPosition);
    float diffuse = max(dot(normal, lightVector), 0.0);

    vec3 viewVector = normalize(cameraPosition - vertexPosition);
    vec3 reflectionVector = normalize(dot(2.0 * normal, lightVector) * normal - lightVector);

    vec3 reflectedLight = reflect(normalize(-lightVector), normal);
    float specular = max(dot(reflectedLight, viewVector), 0.0);
    specular = pow(specular, shine);
    vec4 specularHighlights = specularColor * specular;

    if(hasTexture == 1)
    {
        if(shininess <= 5.0)
        {
            fragmentColor = texture(tex, vertexUVCoord) * ambientColor + texture(tex, vertexUVCoord) * diffuseColor * diffuse;
        }
        else
        {
            fragmentColor = texture(tex, vertexUVCoord) * ambientColor + texture(tex, vertexUVCoord) * diffuseColor * diffuse + specularHighlights;
        }
    }
    else
    {
        if(shininess <= 5.0)
        {
            fragmentColor = ambientColor + diffuseColor * diffuse;
        }
        else
        {
            fragmentColor = ambientColor + diffuseColor * diffuse + specularHighlights;
        }
    }



    if(night == 1)
    {
        fragmentColor = vec4(ambientColor);

        //Spotlight berechnungen
        vec3 lightDirection = normalize(spotLight.position - vertexPosition);
        float theta = dot(lightDirection, normalize(-spotLight.direction)); //cos(rad(theta))
        if(theta > spotLight.outerCone) //Fragment innerhalb des Spotlight-Kreises
        {       
            float NdotLSpot = clamp(dot(normal, lightDirection), 0.0, 1.0);

            // Dmpfung berechnen (Entfernung)
            float distance    = length(spotLight.position - vertexPosition);
            float attenuation = 1.0f / (spotLight.constant + spotLight.linear * distance + spotLight.quadratic * (distance * distance));
      
            //Dmpfung berechnen (Intensitaet outerCone)
            float epsilon = (spotLight.innerCone - spotLight.outerCone);
            float intensity = clamp((theta - spotLight.outerCone) / epsilon, 0.0, 1.0);

            if(hasTexture == 0)
            {
                fragmentColor = fragmentColor + diffuseColor * vec4(spotLight.color * NdotLSpot, 1.0) * intensity * attenuation;
            }
            else
            {
                fragmentColor = fragmentColor + texture(tex, vertexUVCoord) * vec4(spotLight.color * NdotLSpot, 1.0) * intensity * attenuation;
            }
        }
    }

    fragmentColor = clamp(fragmentColor, 0.0, 1.0);
}