#version 410 core

in vec3 vertexPosition;
in vec3 vertexNormal;
in vec2 vertexUVCoord;

uniform sampler2D tex;
vec3 lightPosition = vec3(0.0, 1.0, 0.0);

layout(location=0) out vec4 fragmentColor;

vec3 lightColor = vec3(1.0, 1.0, 1.0);
vec3 ambientColor = vec3(0.0125, 0.0125, 0.0125);

struct SpotLight {
    //Positions- und Beleuchtungs-Angaben
    vec3 position;  
    vec3 direction;
    vec3 color;

    //Gren der inneren und usseren Beleuchtungskreise
    float innerCone;  //cos(rad(wert))
    float outerCone; //cos(rad(wert))

    //Faktoren fr Lichtabschwchung auf Distanz
    float constant;
    float linear;
    float quadratic;
}; 

uniform vec3 spotlightPosition;
uniform vec3 spotlightDirection;
uniform vec3 spotlightColor;
uniform int night;

SpotLight spotLight = SpotLight(spotlightPosition, //position
                                spotlightDirection, //direction
                                spotlightColor,  //color
                                0.976, 0.953, //innerCone = cos(rad(12.5)), outerCone = cos(rad(17.5))
                                1.0, 0.004, 0.000007); //Lichtabschwaechung

void main()
{
    vec3 normal = normalize(vertexNormal);
    vec3 lightVector = normalize(lightPosition);
    float NdotL = clamp(dot(normal, lightVector), 0.0, 1.0);
    fragmentColor = texture(tex, vertexUVCoord) * vec4(ambientColor, 1.0) + texture(tex, vertexUVCoord) * vec4(lightColor, 1.0) * NdotL;

    if(night == 1)
    {
        fragmentColor = vec4(ambientColor, 1.0f);

        //Spotlight berechnungen
        vec3 lightDirection = normalize(spotLight.position - vertexPosition);
        float theta = dot(lightDirection, normalize(-spotLight.direction)); //cos(rad(theta))
        if(theta > spotLight.outerCone) //Fragment innerhalb des Spotlight-Kreises
        {       
        float NdotLSpot = clamp(dot(normal, lightDirection), 0.0, 1.0);

        // Dmpfung berechnen (Entfernung)
        float distance    = length(spotLight.position - vertexPosition);
        float attenuation = 1.0f / (spotLight.constant + spotLight.linear * distance + spotLight.quadratic * (distance * distance));
      
        //Dmpfung berechnen (Intensitaet outerCone)
        float epsilon = (spotLight.innerCone - spotLight.outerCone);
        float intensity = clamp((theta - spotLight.outerCone) / epsilon, 0.0, 1.0);

        fragmentColor = fragmentColor + texture(tex, vertexUVCoord) * vec4(spotLight.color * NdotLSpot, 1.0) * intensity * attenuation;
        }
    }

    fragmentColor = clamp(fragmentColor, 0.0, 1.0);
}