#version 410 core

in vec3 vertexPosition;
in vec3 vertexNormal;
in vec2 vertexUVCoord;

uniform sampler2D tex1;
uniform sampler2D tex2;
uniform sampler2D tex3;

uniform vec3 lightPosition = vec3(-2500.0, 2500.0, 1000.0);
vec4 lightColor = vec4(1.0, 1.0, 1.0, 1.0);

struct SpotLight {
    //Positions- und Beleuchtungs-Angaben
    vec3 position;  
    vec3 direction;
    vec3 color;

    //Gren der inneren und usseren Beleuchtungskreise
    float innerCone;  //cos(rad(wert))
    float outerCone; //cos(rad(wert))

    //Faktoren fr Lichtabschwchung auf Distanz
    float constant;
    float linear;
    float quadratic;
}; 

uniform vec3 spotlightPosition;
uniform vec3 spotlightDirection;
uniform vec3 spotlightColor;
uniform int night;

SpotLight spotLight = SpotLight(spotlightPosition, //position
                                spotlightDirection, //direction
                                spotlightColor,  //color
                                0.976, 0.953, //innerCone = cos(rad(12.5)), outerCone = cos(rad(17.5))
                                1.0, 0.004, 0.000007); //Lichtabschwaechung

layout(location=0) out vec4 fragmentColor;

void main()
{
    fragmentColor = vec4(1.0, 1.0, 1.0, 1.0);
    vec3 normal = normalize(vertexNormal);
    vec3 lightVector = lightPosition - vertexPosition;
    float diffuse = max(dot(normalize(lightVector), normalize(vertexNormal)), 0.0);

    float limit1 = 17.0;
    float limit2 = 32.0;
    float limit3 = 170.0;
    float limit4 = 210.0;
    vec4 texColor = vec4(0.0, 0.0, 0.0, 1.0);
    if(vertexPosition.y <= limit1)
    {
        texColor = fragmentColor * texture(tex1, vertexUVCoord);
    }
    else if(vertexPosition.y > limit1 && vertexPosition.y <= limit2)
    {
        float factor = (vertexPosition.y - limit1) / (limit2 - limit1);
        texColor = fragmentColor * mix(texture(tex1, vertexUVCoord), texture(tex2, vertexUVCoord), factor);
    }
    else if(vertexPosition.y > limit2 && vertexPosition.y <= limit3)
    {
        texColor = fragmentColor * texture(tex2, vertexUVCoord);
    }
    else if(vertexPosition.y > limit3 && vertexPosition.y <= limit4)
    {
        float factor = (vertexPosition.y - limit3) / (limit4 - limit3);
        texColor = fragmentColor * mix(texture(tex2, vertexUVCoord), texture(tex3, vertexUVCoord), factor);
    }
    else
    {
        texColor = fragmentColor * texture(tex3, vertexUVCoord);
    }
    vec4 tex = texColor;
    texColor = texColor * lightColor * diffuse;
    fragmentColor = vec4(texColor.xyz, 1.0);

    if(night == 1)
    {
        fragmentColor = vec4(0.0125, 0.0125, 0.0125, 1.0f);

        //Spotlight berechnungen
        vec3 lightDirection = normalize(spotLight.position - vertexPosition);
        float theta = dot(lightDirection, normalize(-spotLight.direction)); //cos(rad(theta))
        if(theta > spotLight.outerCone) //Fragment innerhalb des Spotlight-Kreises
        {       
        float NdotLSpot = clamp(dot(normal, lightDirection), 0.0, 1.0);

        // Dmpfung berechnen (Entfernung)
        float distance    = length(spotLight.position - vertexPosition);
        float attenuation = 1.0f / (spotLight.constant + spotLight.linear * distance + spotLight.quadratic * (distance * distance));
      
        //Dmpfung berechnen (Intensitaet outerCone)
        float epsilon = (spotLight.innerCone - spotLight.outerCone);
        float intensity = clamp((theta - spotLight.outerCone) / epsilon, 0.0, 1.0);

        fragmentColor = fragmentColor + tex * vec4(spotLight.color * NdotLSpot, 1.0) * intensity * attenuation;
        }
    }

    fragmentColor = clamp(fragmentColor, 0.0, 1.0);
}